#!/bin/bash
###############################################################################################
#                MULTIPATH RESCAN SCRIPT                                                     
#                -----------------------     
#
#When the package failovers from one node to another node,the permission settings of the LUs 
#in the replication group changes from read-only mode to read-write mode at the destination array. 
#In Linux with the Device Mapper Multipath Software, the hosts do not dynamically detect the 
#associated LU permission change and hence the Metrocluster package start fails. 
#
# As a workaround this script rescans the multipath devices
###############################################################################################

	declare -a SCSI_PATH_STR 
	declare -a SCSI_PATH 
	
	SCSI_PATH_STR=`multipath -ll $1`

	SCSI_PATH=`echo "${SCSI_PATH_STR}" | grep sd | sed -n 's/[^0]*0//p' | awk '{print $2}'`
	count=`echo "${SCSI_PATH_STR}" | grep -c "DM multipath kernel driver not loaded"`
	if [[ $count -ne 0 ]]; then
		echo "Multipathing is not enabled "
		exit 2
	fi
	count=`echo "${SCSI_PATH_STR}" | grep -c "lib/udev/scsi_id"`
	if [[ $count -ne 0 ]]; then
		echo "multipath -ll command failed to provide expected output"
		echo "The output provided was:"
		echo "${SCSI_PATH_STR}"
		exit 1
	fi

	if [[ "$SCSI_PATH" != "" ]];then
        MPATH_COUNT=`echo "${SCSI_PATH_STR}" | grep -c ^mpath`
        if [[ "${MPATH_COUNT}" -eq 0 ]];then
            echo "User friendly names are not being used; Flushing with $1."
            multipath -f $1
        else
            MPATH_NAME=`echo "${SCSI_PATH_STR}" | grep $1 | awk '{print $1}'` 
            echo "User friendly names are being used; Flushing with ${MPATH_NAME}"
            multipath -f ${MPATH_NAME}
        fi
		for output in ${SCSI_PATH[@]}
		do
			echo "Disk rescan started for $output"
			echo " echo 1 > /sys/block/$output/device/rescan"
			echo 1 > /sys/block/$output/device/rescan
			echo "Disk rescan complete for $output"
		done

		    multipath -v0
	fi

