####################################################################
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# @(#) Metrocluster 3PARRC Module  Script.
# @(#) Product Name                :  HP Metrocluster 3PARRC for Linux
# @(#) Product Version             :  B.12.00.10
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Metrocluster with 3PAR Remote Copy.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

. /etc/cmcluster.conf

sg_version=`$SGSBIN/cmversion | cut -d "." -f 2-4`
typeset -i all_srv_restart=0
typeset -i disk_not_in_RCVG=0
SG_UTILS=${SGCONF}/scripts/mscripts/utils.sh
export SG_UTILS

# Storing cmviewcl command output.
cmviewcl_output_mc=`cmviewcl -vf line -s config`

# Exporting all the SG variable for the scripts getting called.
export_cmd="export `grep -v '^#' /etc/cmcluster.conf |grep -v '^$'`"
`$export_cmd`

#SCSI_ID command path is changed on RHEL7 & SLES12
if [ -e /usr/lib/udev/scsi_id ]; then
    SCSI_ID="/usr/lib/udev/scsi_id"
    CMCLUSTER=${SGROOT}
else
  if [ -f /etc/redhat-release ]; then
    CMCLUSTER="/usr/local/cmcluster"
    SCSI_ID="/sbin/scsi_id"
    grep 5 /etc/redhat-release 2>&1 > /dev/null
    ver=`/bin/cat /etc/redhat-release | awk -F "[ .]" '{print $7}'`
    if [[ $ver -eq 5 ]];then
        rh5=1
    fi
  else
    CMCLUSTER="/opt/cmcluster"
    SCSI_ID="/lib/udev/scsi_id"
  fi
fi

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}."
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}."
    exit 1
fi


###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

#
# Exporting home directory for root 
#
export HOME="/root"



#######################################################################
# Variable for ssh or cli commands for both arrays and flags
#######################################################################
typeset ssh_cli_cmd_dc1
typeset ssh_cli_cmd_dc2
typeset -i cli_exists=1
typeset check_ssh1_flag=false
typeset check_cli1_flag=false
typeset check_ssh2_flag=false
typeset check_cli2_flag=false
typeset cli_corrupt_flag=false
typeset -i DC1_to_exit=0
typeset -i DC2_to_exit=0

#######################################################################
#
# function check_rpo_threshold() is called by the
# function sg_start_3parrc during metrocluster 3parrc package startup.
# The function calculates the difference between the last syc time and
# the current time, compares the diff with the configured RPO_THRESHOLD.
# The calculation is done only when the package is started with the
# secondary array and if the replication is stopped.
#  
#######################################################################

function check_rpo_threshold
{
  FORCEFLAG=${RC3PAR_DTS_PKG_DIR}FORCEFLAG
  GREP=/bin/grep
  if [[ -f $FORCEFLAG ]]; then
   sg_log 0 "Force flag is set,skipping RPO threshold check"
  else
   if [ $host_type = "DC1" ]; then
      replication_status=`$ssh_cli_cmd_dc1 showrcopy groups ${RC3PAR_DC1_RC_VOLUME_GROUP} -csvtable 2>&1`
    else
      replication_status=`$ssh_cli_cmd_dc2 showrcopy groups ${RC3PAR_DC2_RC_VOLUME_GROUP} -csvtable 2>&1`
    fi

    #Let us check if we are starting with the Secondary Array
    echo ${replication_status} | $GREP ",Secondary," >/dev/null
    if [[ $? -eq 0 ]];then
     #Let us calculate the RPO only in case where RCVG is stopped
     if [[ $(echo ${replication_status} | $GREP ",Stopped," ) ]];then
        lastsyctime=${replication_status##*,}
        lastsyctime=`echo $lastsyctime | cut -d " " -f 1-2`
        currenttime=`date +%Y-%m-%d\ %H:%M:%S`
        lastsync_sec=`date --date="$lastsyctime" +%s`
        current_sec=`date --date="$currenttime" +%s`
        let "diffseconds=$current_sec-$lastsync_sec"
        let "RPO_THRESHOLD_SEC=$RC3PAR_RPO_THRESHOLD*60"
        if [[ $RC3PAR_RPO_THRESHOLD_SEC -le $diffseconds ]];then
            sg_log 0 "ERROR: The Remote Copy replication is stopped and data in the local storage system"
            if [[ $((diffseconds/86400)) -gt 0 ]];then
               sg_log 0 "is not current by $((diffseconds/86400))days:$((diffseconds/3600))hr:$(((diffseconds%3600)/60))min:$((diffseconds%60))sec."
            else
              sg_log 0 "is not current by $((diffseconds/3600))hr:$(((diffseconds%3600)/60))min:$((diffseconds%60))sec."
            fi           
            sg_log 0 "This exceeds the specified RPO_THRESHOLD value of $RC3PAR_RPO_THRESHOLD minutes."  
            sg_log 0 "To start the package forcefully using non current data, use FORCEFLAG file."
            return 1
        fi
     fi
   fi
  fi
}

#######################################################################
#
# function sg_start_3parrc() is called by master control script
# during metrocluster 3parrc package startup.
#
#######################################################################


function sg_start_3parrc
{
    typeset count;
    typeset ret;

    if sg_check_if_reconfig_has_failed ; then        
        sg_log 999 "WARNING: mc3parrc - Not starting Metrocluster 3PARRC Package."        
        return 1    
    fi
    sg_log 0 "Starting Metrocluster 3PARRC Package"
 
    checkifmodused "dts/sc"
    ret=$?
    if [ $ret -eq 0 ]
    then
        sg_log 0 "Site Controller module detected."
        exit 0
    fi

    if [[ ! -e /usr/bin/cli ]]; then
        cli_exists=0
        check_informOs_version ssh
        RC=$?
    else
        check_informOs_version cli
        RC=$?
    fi

    if [[ $RC -ne 0 ]]; then
        exit 1
    fi

    rm -rf $RC3PAR_DTS_PKG_DIR/MC3PARSTARTRCVG

    #Do we need to check RPO
    if [[ $RC3PAR_AUTO_NONCURDATA -eq 1 ]]; then
       if [[ "a${RC3PAR_RPO_THRESHOLD}" != "a" ]] ;then
          #check RPO
           check_rpo_threshold
           if [[ $? -eq 1 ]]; then
             exit 1
           fi
       fi
    fi
       
    if [[ -f ${SGSBIN}/DRCheck3PARRCGrp ]]; then
        ${SGSBIN}/DRCheck3PARRCGrp $SG_PACKAGE_NAME
        ret=$?
    else
        echo "ERROR: Failed to find the DRCheck3PARRCGrp binary in ${SGSBIN}/."
        if sg_check_if_it_is_reconfiguration ; then
           sg_log 999 "ERROR: Failed to find the DRCheck3PARRCGrp binary in ${SGSBIN}/. "
           sg_set_reconfig_failure
        fi
        exit 1
    fi


    if [[ -f  $RC3PAR_DTS_PKG_DIR/MC3PARSTARTRCVG ]]; then
        sg_log 0 "Trying to start 3PAR Remote Copy volume group in background."
        ${SGSBIN}/3parrc/MC3PARStartRCVG $SG_PACKAGE_NAME &
    fi



    if [[ -f  $RC3PAR_DTS_PKG_DIR/MC3PARSTARTRCVG ]]; then
        sg_log 0 "Trying to start 3PAR Remote Copy volume group in background."
        ${SGSBIN}/3parrc/MC3PARStartRCVG $SG_PACKAGE_NAME &
    fi
 

#    if [[ "$RC3PAR_CLUSTER_TYPE" = "continental" && $ret -eq 3 ]]; then
#    fi

    if [[ $ret -eq 3 ]]; then
    echo "export SFC_MC_RESCAN_REQUIRED=\"1\"" >$SG_SRC_FRM_CHILD
    do_drif
    if [[ $? -eq 1 ]]
    then
       if sg_check_if_it_is_reconfiguration ; then        
          sg_log 999 "ERROR: Failed to clear the volume manager tag."        
          sg_set_reconfig_failure        
       fi
       return 1
    fi

    typeset wwids=""
    if [ $host_type = "DC1" ]; then
        echo $ssh_cli_cmd_dc1| grep -q "cli"
        if [[ $? -eq 0 ]]; then
            wwids=`$ssh_cli_cmd_dc1 showvv -host ${host_name} -rcopy -showcols RcopyGroup,VV_WWN | awk '/'${RC3PAR_DC1_RC_VOLUME_GROUP}'/ {print $2}'`
        else
            echo $ssh_cli_cmd_dc1| grep -q "ssh"
            if [[ $? -eq 0 ]]; then
                wwids=`$ssh_cli_cmd_dc1 showvv -host ${host_name} -rcopy -showcols RcopyGroup,VV_WWN | awk '/'${RC3PAR_DC1_RC_VOLUME_GROUP}'/ {print $2}'`
            fi
        fi
    else
        echo $ssh_cli_cmd_dc2| grep -q "cli"
        if [[ $? -eq 0 ]]; then
            wwids=`$ssh_cli_cmd_dc2 showvv -host ${host_name} -rcopy -showcols RcopyGroup,VV_WWN | awk '/'${RC3PAR_DC2_RC_VOLUME_GROUP}'/ {print $2}'`
        else
            echo $ssh_cli_cmd_dc1| grep -q "ssh"
            if [[ $? -eq 0 ]]; then
                wwids=`$ssh_cli_cmd_dc2 showvv -host ${host_name} -rcopy -showcols RcopyGroup,VV_WWN | awk '/'${RC3PAR_DC2_RC_VOLUME_GROUP}'/ {print $2}'`
            fi
        fi
    fi

    for disk in ${wwids[@]}; do
        sg_log 0 "Disk rescan is started for $disk"
        disk=`echo $disk | awk '{print tolower($0)}'`
        ${CMCLUSTER}/bin/3parrc/mc3parrc_multipath_rescan.sh 3${disk}
        sg_log 0 "Re-scan of  multipath devices returned $?."
        sg_log 0 "Disk rescan is complete for  $disk."
    done

    sleep 10 # sleeping for rescan to take effect

    for vg in ${SG_VG[*]}
                do
                        typeset hostid=""

                        hostid=$(vgs --noheadings -o +tags $vg | awk  '{print $8}')
                        if [[ "$hostid" = *,* ]]; then
                        sg_log 0 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
                        sg_log 0 "There cannot be more than one tag."
                        sg_log 0 "Remove the tags using the command:"
                        for i in ${hostid}; do
                            sg_log 0 "\"vgchange --deltag $i $vg\"."
                        done
                        if sg_check_if_it_is_reconfiguration ; then
                            sg_log 999 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
                            sg_set_reconfig_failure
                        fi
                        return 1
                        fi

                        if [[ "$hostid" != "" ]] ; then
                        #       cmexec $hostid vgchange -a n $vg
                                vgchange --deltag $hostid $vg
                        fi
                done

       ret=0
    fi

    exit $ret
}

#######################################################################
#
# function sg_stop_3parrc() is called by master control script
# during metrocluster 3parrc package stop.
#
#######################################################################


function sg_stop_3parrc
{
    sg_log 0 "Stopping Metrocluster 3PARRC Package"
    rm -f $RC3PAR_DTS_PKG_DIR/MC3PARSTARTRCVG
    # no op
    to_exit=0

}

###############################################################################
# This function checks whether given module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - module is used
#          1 - module is not used
#
###############################################################################

function checkifmodused
{
   typeset used_module=$1;
   let exit_val=1
   typeset module_name;
   typeset module;

   module_name=`echo "${SG_MODULE_NAME[@]}"`

   for module in $module_name
   do
      if [ "$module" = "$used_module" ]
      then
         let exit_val=0
         break
      fi
   done

   return $exit_val

}


######################################################################
#
# function to do drif for 3par 
#
#####################################################################

function do_drif
{
# not needed for Linux and hence commenting
#        checklvmused ${SG_VG[*]}
#        vgret=$?;
#        if [ $vgret -eq  0 ]
#        then
#            reset_lvms
#        if [[ $? -ne 0 ]]
#        then
#        return 1
#        fi
#        elif [ $vgret -eq 2 ]
#        then
#            reset_slvms
#            if [[ $? -ne 0 ]]
#            then
#                return 1
#            fi
#        fi

        checkvxvmused ${SG_VXVM_DG[*]}
        if [ $? -ne 1 ]
        then
            reset_vxvms
            if [[ $? -ne 0 ]]
            then
                return 1
            fi
        fi

#        checkcvmused ${SG_CVM_DG[*]}
#        if [ $? -ne 1 ]
#        then
#        reset_cvm
#            if [[ $? -ne 0 ]]
#            then
#                return 1
#            fi
#        fi

}

###############################################################################
# This function checks details about VMFS disks specified.
# Internal Use only
#
#
# Arguments: $1: Node Name
#
#
# Returns: 0 if SUCCESS and value returned is Node UUID of the 
#          Node Name passed to the function.
###############################################################################
function get_uuid_from_node_name
{
    uuid=""
    node_name=$1
    uuid=`echo "$cmviewcl_output_mc" | grep $node_name | awk -F= '/^node:[^|]*\|uuid=/ {print $2}'`
    echo $uuid
}


###############################################################################
# This function checks details about VMFS disks specified.
# Internal Use only
#
#
# Arguments: $1: RW Site
#
#
# Returns: 0 if SUCCESS
#          1 if disk specified in package is not part of RCVG
###############################################################################
function verify_vmfs_details
{        
    typeset eligible_nodes_vmfs=""
    typeset ro_nodes=""
    typeset wwids_rw=""
    typeset wwids_ro=""
    typeset rcvg_rw=""
    typeset rcvg_ro=""
    typeset RW_site=$1
    typeset -i disk_flag=0

    if [[ $RW_site == "DC1" ]]
    then
        eligible_nodes_vmfs=(${RC3PAR_DC1_NODE_LIST})
        ro_nodes=(${RC3PAR_DC2_NODE_LIST})
    else
        eligible_nodes_vmfs=(${RC3PAR_DC2_NODE_LIST})
        ro_nodes=(${RC3PAR_DC1_NODE_LIST})
    fi

    
    # Pass the eligible nodelist to Master Control Script
    echo "export SFC_MC_READ_WRITE_NODELIST=\"${eligible_nodes_vmfs[*]}\"" >$SG_SRC_FRM_CHILD
    echo "export SFC_MC_READ_ONLY_NODELIST=\"${ro_nodes[*]}\"" >>$SG_SRC_FRM_CHILD
    
    
        cur_hostname=${SG_HOST_NAME%%.*}
        # Verification is done only from one node.
        if [[ ${cur_hostname} = ${eligible_nodes_vmfs[0]} ]]; then
            eligible_node_uuid=$(get_uuid_from_node_name ${eligible_nodes_vmfs[0]})
            ro_node_uuid=$(get_uuid_from_node_name ${ro_nodes[0]})
            vmfs_details=$($SGSBIN/cmvmdiskquery -n $eligible_node_uuid -n $ro_node_uuid)
            if [[ $? -ne 0 ]]; then
                    echo "ERROR: Unable to query the Host"
                    exit 1
            fi
            
            # Verifying the RW site.
            if [[ $RW_site == "DC1" ]]
            then
                wwids_rw=`$ssh_cli_cmd_dc1 showvv -p -rcopygroup ${RC3PAR_DC1_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
                rcvg_rw="${RC3PAR_DC1_RC_VOLUME_GROUP}"
                wwids_ro=`$ssh_cli_cmd_dc2 showvv -p -rcopygroup ${RC3PAR_DC2_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
                rcvg_ro="${RC3PAR_DC2_RC_VOLUME_GROUP}"
            else
                wwids_rw=`$ssh_cli_cmd_dc2 showvv -p -rcopygroup ${RC3PAR_DC2_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
                rcvg_rw="${RC3PAR_DC2_RC_VOLUME_GROUP}"
                wwids_ro=`$ssh_cli_cmd_dc1 showvv -p -rcopygroup ${RC3PAR_DC1_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
                rcvg_ro="${RC3PAR_DC1_RC_VOLUME_GROUP}"
            fi

            # Verifying that all the disks in the RCVG is part of the Package.
            for disk_wwid in ${wwids_rw}
            do
                disk_flag=0
                for ds_name in ${SG_DATASTORE_NAME[*]}
                do
                    echo "$vmfs_details"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|grep -i "$disk_wwid" >/dev/null 2>&1
                    if [[ $? -eq 0 ]]; then
                        disk_flag=1
                    fi
                done
                if [[ $disk_flag -eq 0 ]];then
                    echo "WARNING: mc3parrc The disk with WWID $disk_wwid, configured as part of the Remote Copy Volume Group $rcvg_rw"
                    echo "      is not part of any of the Datastore specified in the package."
                fi
            done

            # Verifying that all the disk in the Data Store is part of RCVG
            disk_flag=0
            for ds_name in ${SG_DATASTORE_NAME[*]}
            do
                ds_wwids=`echo "${vmfs_details}"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|cut -d '=' -f2`
                for wwid in ${ds_wwids}
                do
                    echo "${wwids_rw}"|grep -i "${wwid##*.}" >/dev/null 2>&1                
                    if [[ $? -ne 0 ]]; then
                        echo "ERROR: The disk with WWID ${wwid##*.}, used for the Datastore ${ds_name} is not part of the Remote Copy Volume Group $rcvg_rw"
                        disk_flag=1
                    fi
                done
            done
            if [[ $disk_flag -eq 1 ]];then
                exit 1
            fi
            
            # Verifying that all the disks in the RCVG is part of the Package.
            for disk_wwid in ${wwids_ro}
            do
                echo "$vmfs_details"|grep -i "VirtualMachine:${ro_node_uuid}|Disk:" |grep -i $disk_wwid >/dev/null 2>&1            
                #echo "$vmfs_details" | awk -F "[:|=]" '(($1 == "VirtualMachine") \
                #         && ($2 == "'${ro_node_uuid}'") && ($3 == "Disk")) \
                #         {print $4}' | grep -i $disk_wwid >/dev/null 2>&1
                if [[ $? -ne 0 ]]; then
                    echo "WARNING: mc3parrc The disk with WWID $disk_wwid, configured as part of the Remote Copy Volume Group $rcvg_ro"
                    echo "       in the Read only site is not available to the host."
                fi
            done  
        fi
    
}

###############################################################################
# This function checks whether the volume manager is lvm or slvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - lvm is used
#          1 - lvm is not used
#          2 - slvm is used
#
###############################################################################

function checklvmused
{
   if [[ ${SG_VG[*]} != "" ]]
   then

      if [[ ${SG_VGCHANGE_CMD} = "" ]]
      then
         sg_log 0 "\tERROR: Function checklvmused."
         sg_log 0 "\tERROR: LVM Activation Command can not be empty while using LVM group."
         exit 1
      fi

      if [[ "${SG_VGCHANGE_CMD#*-a}" = " s" ]]
      then
         return 2;
      fi

      return 0
   fi
   return 1
}



#########################################################################
#
# function to change cluster id of LVM VGs
#
########################################################################

function reset_lvms
{

for vg in "${SG_VG[@]}"
do
    sg_log 5 "Executing: vgchange -c n $vg"
    /sbin/vgchange -c n $vg >/dev/null
    if [[ $? -ne 0 ]]
    then
        sg_log 0 "ERROR: Failed to execute \"vgchange -c n ${vg}\"."
        return 1
    fi
    sg_log 5 "Executing: vgchange -c y $vg"    
    /sbin/vgchange -c y $vg >/dev/null
    if [[ $? -ne 0 ]]
    then
    sg_log 0 "ERROR: Failed to execute \"vgchange -c y ${vg}\"."
    return 1
    fi
    sg_log 0 "Cluster ID change for LVM VG ${vg} is successful."
done

}

#########################################################################
#
# function to change the host id in vxvm volumes
#
#########################################################################

function reset_vxvms
{
for vg in " ${SG_VXVM_DG[@]}"
do
 vxdg -tfC import $vg
done
}

####################################################################
# Checks whether a device is a DM multipath device.
# Any device in format /dev/mapper/XXXX is a multipath device.
# On RHEL 5.x, device can also be in format /dev/mpath/mpathX
# For udev, we do a readlink and if its in dm-x format, it is a
# multipath device.
####################################################################

function check_if_multipath
{
    typeset device=$1

    echo $device | grep -q -e "/dev/mapper/" -e "/dev/mpath/"
    if [ $? -eq 0 ]
    then
        return 0
    else
        udev_tmp=`readlink $device`
        if [ $? -eq 0 ]
        then
            echo $udev_tmp | grep -q "dm-"
            if [ $? -eq 0 ]
            then
                return 0
            fi
        fi
        return 1
    fi
}

######################################################################
#
# function verification_after_pkgstart is called when "cmcheckconf -v"
# is executed.This will be ideally called when the pkg is already up
# and running.
#
#######################################################################


function verification_after_rc3par_pkgstart
{
     GREP=/bin/grep
     FORCEFLAG=${RC3PAR_DTS_PKG_DIR}FORCEFLAG
     ret=0

     if [[ ${host_type} = "DC1" ]]; then
       #In DC1, check if the password file exists
       if [[ ! -e ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf ]];then
                echo "WARNING: mc3parrc - The password file ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf \
                is not found."
       else
         #In DC1, check if the repilcation takes place
         replication_status=`$ssh_cli_cmd_dc1 showrcopy groups ${RC3PAR_DC1_RC_VOLUME_GROUP} | $GREP "Stopped" 2>&1`
         if [[ $(echo ${replication_status} | $GREP -i "Stopped" ) ]];then
            if [[ $RC3PAR_AUTO_NONCURDATA -eq 1 ]]; then
                if [[ "a${RC3PAR_RPO_THRESHOLD}" != "a" ]] ;then
                  #check RPO
                   check_rpo_threshold
                   if [[ $? -ne 1 ]]; then
                      echo "WARNING: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC1_RC_VOLUME_GROUP} is stopped."
                    else
                     ret=1
                   fi
                else
                   echo "WARNING: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC1_RC_VOLUME_GROUP} is stopped."                
                fi
            else
              echo "ERROR: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC1_RC_VOLUME_GROUP} is stopped."
              ret=1
            fi
         fi
       fi

     else
       #In DC2, check if the password file exists
       if [[ ! -e ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf ]];then
                echo "WARNING: mc3parrc - The password file ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf \
                is not found."
       else
        #In DC2, check if the repilcation takes place
         replication_status=`$ssh_cli_cmd_dc2 showrcopy groups ${RC3PAR_DC2_RC_VOLUME_GROUP} | $GREP "Stopped" 2>&1`
         if [[ $(echo ${replication_status} | $GREP -i "Stopped" ) ]];then
            if [[ $RC3PAR_AUTO_NONCURDATA -eq 1 ]]; then
                if [[ "a${RC3PAR_RPO_THRESHOLD}" != "a" ]] ;then
                  #check RPO
                   check_rpo_threshold
                   if [[ $? -ne 1 ]]; then
                     echo "WARNING: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC2_RC_VOLUME_GROUP} is stopped."
                    else
                     ret=1
                   fi
                else
                   echo "WARNING: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC2_RC_VOLUME_GROUP} is stopped."                   
                fi
            else
              echo "ERROR: mc3parrc - The Remote Copy replication for the RCVG ${RC3PAR_DC2_RC_VOLUME_GROUP} is stopped."
              ret=1                
            fi
         fi
       fi
     fi

     #check if the FORCEFLAG file exists
     if [[  -f $FORCEFLAG ]];then
        echo "WARNING: mc3parrc - The FORCEFLAG $FORCEFLAG, is present in the package directory."
     fi
     
     if [[ $ret -ne 0 ]]; then
         to_exit=1
     fi

}

#################################################################################
# This function checks whether the disks belonging to a LVM are being
# replicated.
# Internal Use only
# Arguments: None
# Returns: None
###############################################################################

function lvm_3par_disks_replicated
{
   for VG in ${SG_VG[@]};do
    unset list_of_devices
    list_of_devices=$(vgdisplay -v $VG 2> /dev/null | fgrep "PV Name" | 
                       grep -v "unknown device" | awk '{print $NF}')
    mc3parrc_get_disk_info ${VG} "lvm"
   done
}

##################################################################################
# This function checks whether the disks belonging to a VXVM diskgroup are
# being replicated.
# Internal Use only
# Arguments: None
# Returns: None
###############################################################################

function vxvm_3par_disks_replicated
{
    for vg in ${SG_VXVM_DG[@]}; do
        unset list_of_devices
        list_of_devices=$(vxdisk -s list | awk -v vg=$vg '
                         /^Disk:/   { disk = $2 }
                         /^dgname:/ { if (vg == $2) { print disk; } }')
        list_of_devices=`echo ${list_of_devices} | sed 's/\n/ /g'`
        mc3parrc_get_disk_info ${vg} "vxvm"
    done
}

##################################################################################
# This function checks whether the disk passed as an argument is being replicated.
# Internal Use only
# Arguments:  LVM/VXVM groupname, lvm/vxvm
# Returns:  None
#
###############################################################################
function mc3parrc_get_disk_info
{
   vg=$1
   if [[ "$2" = "lvm" ]]; then
       type_of_dg="Volume Group"
   else
      type_of_dg="Disk Group"
   fi
 
   no_of_devices=0
   no_of_devices_not_in_vg=0
   disks_not_in_vg=""
   for DEV in ${list_of_devices[@]};do
     if [[ "$2" = "vxvm" ]]; then
       if [[ $DEV != sd* ]]; then
         #VxVM is using  a internal format to identify the disks
         #cannot proceed with the check if the disk are in this format
         #or
         #the devices are in enclosure based format, so let us continue
         to_exit=0
         continue
        fi
     fi
     no_of_devices=`expr $no_of_devices + 1`

    check_if_multipath $DEV
        if [[ $? -eq 0 ]]; then
           #if multipath use dmsetup to get the wwn
           disk_wwn_pkgvg=`/sbin/dmsetup info $DEV | grep UUID | sed 's/-/ /g' | awk '{print $NF}' \
                    | awk '{print substr($0,2)}'|awk '{print toupper($0)}'`
        else
           #For non multipath device use scsi_id command after stripping the partition number if any
           disk=`/bin/basename $DEV | sed 's/[0-9].*//'`
       #In RH5 scsi_id has different options and also it does not work on multipath devices
       if [[ $rh5 -eq 1 ]];then
                  SCSI_ID="/lib/udev/scsi_id -g -s /block/"
               disk_wwn_pkgvg=`$SCSI_ID$disk | awk '{print substr($0,2)}'|awk '{print toupper($0)}'`
       else
        if [[ "$2" = "vxvm" ]]; then
               disk_wwn_pkgvg=`$SCSI_ID --whitelisted /dev/$disk | awk '{print substr($0,2)}'|awk '{print toupper($0)}'`
            else
                 disk_wwn_pkgvg=`$SCSI_ID --whitelisted /dev/$disk | awk '{print substr($0,2)}'|awk '{print toupper($0)}'`
            fi
           fi
    fi
 
     found=`echo $storage_wwn_in_array | grep $disk_wwn_pkgvg 2>/dev/null`
     if [[ $? -ne 0 ]];then
        #The disk is not part of the RCVG
        disk_not_in_RCVG=1
        no_of_devices_not_in_vg=`expr $no_of_devices_not_in_vg + 1`
        disks_not_in_vg="$disks_not_in_vg $DEV"
     fi
    done

    if [[ $disk_not_in_RCVG -eq 1 ]];then
       echo "ERROR: Disks ${disks_not_in_vg} in the ${type_of_dg} ${vg} do not belong to Remote Copy Volume Group ${RC_VOLUME_GROUP} configured in this Metrocluster package."
       to_exit=1
    fi
#    if [[ $no_of_devices_not_in_vg -lt $no_of_devices ]];then
    #Warn if any disk in the RCVG is not used in the VG  
#       echo "WARNING: mc3parrc - The disks in the Remote Copy Volume Group ${RC_VOLUME_GROUP} is not part of ${type_of_dg} $vg configured in this Metrocluster package. Ignore this warning if this is being done intentionally."

#           to_exit=0
#         fi
}

#######################################################################
#
# function sg_validate_3parrc() is called by master control script
# during metrocluster 3parrc configuration validation.
#
#######################################################################

function sg_validate_3parrc
{
    typeset RW_site=""
    sg_log 0 "Validating Metrocluster Configuration"
    if [[ ! -d $RC3PAR_DTS_PKG_DIR ]]; then
        mkdir -p -m 755 $RC3PAR_DTS_PKG_DIR
        if [ $? -eq 0 ]; then
           sg_log 0 "Creation of \"$RC3PAR_DTS_PKG_DIR\" successful."
        else
           echo "ERROR: Failed to create \"$RC3PAR_DTS_PKG_DIR\". "
           to_exit=1
        fi
    fi
    
    if [[ "$host_type" = "NA" ]]; then
        sg_log 0 "ERROR: Node mentioned is not present in DC1_NODE_LIST or DC2_NODE_LIST"
        exit 1
    fi
    

    if [[ ! -e /usr/bin/cli ]]; then
        cli_exists=0
        check_informOs_version ssh
        RC=$?
    else
        check_informOs_version cli
        RC=$?
    fi

    if [[ $RC -ne 0 ]]; then
       exit 1
    fi

#Verify if paswordless connection using password file  is configured.


#check if the DC1 and DC2 volume groups are part of the same replication pair.
#Ensure that either dc2's volume group  is a substring of dc1's volume group or vice versa
        if [[ ${host_type} = "DC1" ]]; then
           dc2_node=`echo $RC3PAR_DC2_NODE_LIST| awk '{print $1}'`
           dc1_system_id=`$ssh_cli_cmd_dc1 showsys -d  -csvtable | grep "System ID" | awk -F',' '{ print $2; }' 2>&1`
           rc1=$?
           dc2_system_id=`$ssh_cli_cmd_dc2 showsys -d  -csvtable | grep "System ID" | awk -F',' '{ print $2; }' 2>&1`
           rc2=$?
        elif [[ ${host_type} = "DC2" ]]; then
           dc1_node=`echo $RC3PAR_DC1_NODE_LIST| awk '{print $1}'`
           dc1_system_id=`$ssh_cli_cmd_dc1 showsys -d  -csvtable | grep "System ID" | awk -F',' '{ print $2; }' 2>&1`
           rc1=$?
           dc2_system_id=`$ssh_cli_cmd_dc2 showsys -d  -csvtable | grep "System ID" | awk -F',' '{ print $2; }' 2>&1`
           rc2=$?
        fi
        if [[ $rc1 != 0 ]] || [[ $rc2 != 0 ]]; then
            exit 1
        fi
        
        #convert the system id to decimal if it is in hex value.
        dc1_system_id=$((dc1_system_id))
        dc2_system_id=$((dc2_system_id))

        echo ${RC3PAR_DC1_RC_VOLUME_GROUP} | grep ".r${dc2_system_id}" 2>&1  >/dev/null
        flag1=$?
        echo ${RC3PAR_DC2_RC_VOLUME_GROUP} | grep ".r${dc1_system_id}" 2>&1  >/dev/null
        flag2=$?
        echo ${RC3PAR_DC2_RC_VOLUME_GROUP} | grep ".r${dc2_system_id}" 2>&1  >/dev/null
        flag3=$?
        echo ${RC3PAR_DC1_RC_VOLUME_GROUP} | grep ".r${dc1_system_id}" 2>&1  >/dev/null
        flag4=$?

        if [[ ($flag1 -eq 0) && ($flag3 -eq 0) ]]; then
                dc2_rc_volume_group_subs=${RC3PAR_DC1_RC_VOLUME_GROUP}.r$dc2_system_id
                dc1_rc_volume_group_subs=${RC3PAR_DC2_RC_VOLUME_GROUP}.r$dc2_system_id
                if [[ (${dc2_rc_volume_group_subs} = ${RC3PAR_DC1_RC_VOLUME_GROUP}) || (${dc1_rc_volume_group_subs} = ${RC3PAR_DC2_RC_VOLUME_GROUP}) ]]; then
                echo "Success"  >/dev/null
                else
                echo "ERROR: DC2 RC VOLUME GROUP ${RC3PAR_DC2_RC_VOLUME_GROUP} is not the name of Remote Copy volume group ${RC3PAR_DC1_RC_VOLUME_GROUP} on the storage system ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}."
                to_exit=1
                fi
        elif [[ ($flag2 -eq 0) && ($flag4 -eq 0) ]]; then
                dc2_rc_volume_group_subs=${RC3PAR_DC1_RC_VOLUME_GROUP}.r$dc1_system_id
                dc1_rc_volume_group_subs=${RC3PAR_DC2_RC_VOLUME_GROUP}.r$dc1_system_id
                if [[ (${dc2_rc_volume_group_subs} = ${RC3PAR_DC2_RC_VOLUME_GROUP}) || (${dc1_rc_volume_group_subs} = ${RC3PAR_DC1_RC_VOLUME_GROUP}) ]]; then
                echo "Success" >/dev/null
                else
                echo "ERROR: DC2 RC VOLUME GROUP ${RC3PAR_DC2_RC_VOLUME_GROUP} is not the name of Remote Copy volume group ${RC3PAR_DC1_RC_VOLUME_GROUP} on the storage system ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}."
                to_exit=1
                fi
        elif [[ $flag1 -eq 0 ]]; then
                dc2_rc_volume_group_subs=${RC3PAR_DC2_RC_VOLUME_GROUP}.r$dc2_system_id
                if [[ ${dc2_rc_volume_group_subs} != ${RC3PAR_DC1_RC_VOLUME_GROUP} ]]; then
                        echo "ERROR: DC2 RC VOLUME GROUP ${RC3PAR_DC2_RC_VOLUME_GROUP} is not the name of Remote Copy volume group ${RC3PAR_DC1_RC_VOLUME_GROUP} on the storage system ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}."
                to_exit=1
                fi
        elif [[ $flag2 -eq 0 ]]; then
                dc1_rc_volume_group_subs=${RC3PAR_DC1_RC_VOLUME_GROUP}.r$dc1_system_id
                if [[  ${dc1_rc_volume_group_subs} != ${RC3PAR_DC2_RC_VOLUME_GROUP} ]]; then
                         echo "ERROR: DC2 RC VOLUME GROUP ${RC3PAR_DC2_RC_VOLUME_GROUP} is not the name of Remote Copy volume group ${RC3PAR_DC1_RC_VOLUME_GROUP} on the storage system ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}."
                to_exit=1
                fi
        else
                echo "Invalid Remote Copy volume group name"
                to_exit=1
        fi

        if [[ ${RC3PAR_DC1_RC_VOLUME_GROUP} = ${RC3PAR_DC2_RC_VOLUME_GROUP} ]]; then
                echo "ERROR: Both DC1 RC VOLUME GROUP and DC2 RC VOLUME GROUP cannot have the same value."
                to_exit=1
        fi

#Check if the volume exists.
        if [[ ${host_type} = "DC1" ]]; then
                 check_rvg_command=`$ssh_cli_cmd_dc1 showrcopy groups ${RC3PAR_DC1_RC_VOLUME_GROUP} -csvtable 2>&1`
                 site_type=`echo "${check_rvg_command}"|grep ${RC3PAR_DC1_RC_VOLUME_GROUP}|cut -d',' -f4 `
                 if [[ $site_type == "Primary" ]]
                 then
                      RW_site="DC1"
                 else
                      RW_site="DC2"
                 fi
        else
                 check_rvg_command=`$ssh_cli_cmd_dc2 showrcopy groups ${RC3PAR_DC2_RC_VOLUME_GROUP} -csvtable 2>&1`
                 site_type=`echo "${check_rvg_command}"|grep ${RC3PAR_DC2_RC_VOLUME_GROUP}|cut -d',' -f4 `
                 if [[ $site_type == "Primary" ]]
                 then
                      RW_site="DC2"
                 else
                      RW_site="DC1"
                 fi
        fi

    if [[ $(echo ${check_rvg_command} | grep -i "does not exist on the system" ) != "" ]];then
        if [[  ${host_type} = "DC1" ]]; then
            echo "ERROR: The Remote Copy volume group ${RC3PAR_DC1_RC_VOLUME_GROUP} does not exist on the storage system ${RC3PAR_DC1_STORAGE_SYSTEM_NAME}."
        else
            echo "ERROR: The Remote Copy volume group ${RC3PAR_DC2_RC_VOLUME_GROUP} does not exist on the storage system ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}."
        fi
        exit 1
    fi

#check if it is Peer Persistence environment
    if [[ $(echo ${check_rvg_command} | grep "path_management" ) != "" ]];then
       cmviewcl_cmd="$SGSBIN/cmviewcl -vf line -s config"
       node_type=`$cmviewcl_cmd | grep -w ${host_nameWithoutDomain} | awk -F= '/^node:[^|]*\|virt_tech=/ {print $2}'`
       if [[ ${node_type} = "vmware" ]];then
          if [[ ${#SG_VMDK_FILE_NAME[*]} = 0 ]]; then
            echo "ERROR: Metrocluster supports Peer persistence only with VMFS disks or VMFS virtual RDM disks."
            to_exit=1
          fi
       fi
    fi

    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
        
             verify_vmfs_details "$RW_site"
             if [ $? -ne 0 ]; then
                 echo "ERROR: Failed to validate VMFS details mentioned in the package"
                 to_exit=1
             fi
        
    else
    #check if the disk belonging to the VG is of this replication group
        if [[ ${host_type} = "DC1" ]]; then
           #get the wwn from the respective array
           storage_wwn_in_array=`$ssh_cli_cmd_dc1 showvv -rcopygroup ${RC3PAR_DC1_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
           RC_VOLUME_GROUP=${RC3PAR_DC1_RC_VOLUME_GROUP}
        else
           storage_wwn_in_array=`$ssh_cli_cmd_dc2 showvv -rcopygroup ${RC3PAR_DC2_RC_VOLUME_GROUP} -showcols VV_WWN -csvtable 2>&1 | grep -v "^VV_WWN" | grep -v "^\-\-\-\-\-\-"`
          RC_VOLUME_GROUP=${RC3PAR_DC2_RC_VOLUME_GROUP}
        fi

        checklvmused ${SG_VG[*]}
        ret=$?
        if (( $ret == 0 )) || (( $ret == 2 )) ; then
             lvm_3par_disks_replicated
        fi

        checkvxvmused ${SG_VXVM_DG[*]}
        if [ $? -ne 1 ]; then
           vxvm_3par_disks_replicated
        fi
    fi

#check if the target names are fine
    if [[ ${host_type} = "DC1" ]]; then
        echo "${check_rvg_command}" | grep "${RC3PAR_DC1_RC_VOLUME_GROUP},${RC3PAR_DC1_RC_TARGET_FOR_DC2}" 2>&1  >/dev/null
        if [[ $? != "0" ]]; then
        echo "ERROR: The DC1 Remote Copy target name for DC2 ${RC3PAR_DC1_RC_TARGET_FOR_DC2} does not match the actual value $(echo "${check_rvg_command}" | grep ${RC3PAR_DC1_RC_VOLUME_GROUP} | awk -F [,] '{ print $2 }' )."
        to_exit=1
        fi
    else
    echo "${check_rvg_command}" | grep "${RC3PAR_DC2_RC_VOLUME_GROUP},${RC3PAR_DC2_RC_TARGET_FOR_DC1}" 2>&1 >/dev/null
        if [[ $? != 0 ]]; then
            echo "ERROR: The DC2 Remote Copy target name for DC1 ${RC3PAR_DC2_RC_TARGET_FOR_DC1} does not match the actual value $(echo "${check_rvg_command}" | grep ${RC3PAR_DC2_RC_VOLUME_GROUP} | awk -F [,] '{ print $2 }' )."
            to_exit=1
        fi
    fi

    

        sg_ver=`${CMCLUSTER}/bin/cmversion | cut -d "." -f 2-4`
        if [[ "$sg_ver" > "11.20.20" || "$sg_ver" = "11.20.20" ]]; then
            if [[ $SG_PRIV_CLUSTER_VERIFICATION == y ]];then
                #Let us do additional checks
                 verification_after_rc3par_pkgstart
            fi
        fi

#check if RPO_THRESHOLD is set without setting AUTO_NONCURDATA to one
    if [[ "a${RC3PAR_RPO_THRESHOLD}" != "a" ]] ;then 
       if [[ $RC3PAR_AUTO_NONCURDATA -eq 0 ]]; then
        echo "ERROR: The Attribute RPO_THRESHOLD should be set only when the parameter AUTO_NONCURDATA is set to 1(one)."
        to_exit=1        
       fi      
    fi
       
       
# check whether notification types are configured when the RCVG Monitor service is configured
   for cmd in "${SG_SERVICE_CMD[@]}"
   do
     echo $cmd | grep "$SGSBIN/DR3PARRCMon $SG_PACKAGE_NAME" >> /dev/null
     if [[ $? -eq 0 ]]
     then
        service_name=${SG_SERVICE_NAME[$i]}
        break
     fi
     i=$i+1
   done
   if [[ "a${service_name}" != "a" ]];then
     if [[ "a${RC3PAR_DR_NOTIFICATION_CHOICES}" = "a" ]];then
        echo "ERROR: Remote Copy Volume Group Monitor requires the parameter"
    echo "dts/3parrc/DR_NOTIFICATION_CHOICES. Please specify appropriate values for it."    
    to_exit=1
        return $to_exit
     fi
   fi
   

# check if the notification types specified are valid
    for notif in $RC3PAR_DR_NOTIFICATION_CHOICES
        do
            if [[ $notif != "EMAIL" && $notif != "SYSLOG" && $notif != "CONSOLE" && $notif != "OPC" ]];then
               echo "ERROR: Invalid Notification Type $notif specified."
               to_exit=1
               return $to_exit
            fi    
        done

#check if EMAIL is mentioned in the Notification Choices and email 
#addresses are mentioned as part of Package configuration.

    echo "${RC3PAR_DR_NOTIFICATION_CHOICES}" | grep "EMAIL" >>/dev/null
    if [[ $? -eq 0 ]]; then
       if [[ -z ${SG_EMAIL_ID[@]} ]]; then 
             echo "ERROR: EMAIL Notification type is specified but EMAIL addresses are not configured."
             to_exit=1
       fi
    fi
    return $to_exit
}

function check_ssh_connection
{
    
    if [[ "$1" = "DC1" ]]; then
        ssh -l ${RC3PAR_DC1_STORAGE_SYSTEM_USER} -q -o ConnectTimeout=1 -o StrictHostKeyChecking=no -o BatchMode=yes ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} showsshkey > /dev/null 2>&1
        ret_val=$?
        if [[ $ret_val -ne 0 ]]; then
            if [[ $cli_exists -eq 0 ]]; then
                sg_log 0 "ERROR: Could not access the 3PAR storage array ${RC3PAR_DC1_STORAGE_SYSTEM_NAME}. Neither Passwordless SSH is configured nor CLI is found."
                sg_log 0 "If Inform OS version is greater than 3.3.1, then Passwordless SSH needs to be configured."
                sg_log 0 "If Inform OS version is lesser than 3.3.1, then install CLI client and create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary."
                return 1
            else
                DC1_SSH_ERROR="ERROR: Passworldless SSH not set for ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} array with user ${RC3PAR_DC1_STORAGE_SYSTEM_USER} from system $host_nameWithoutDomain ."
                return 1 
            fi
        fi
    elif [[ "$1" = "DC2" ]]; then
        ssh -l ${RC3PAR_DC2_STORAGE_SYSTEM_USER} -q -o ConnectTimeout=1 -o StrictHostKeyChecking=no -o BatchMode=yes ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} showsshkey > /dev/null 2>&1
        ret_val=$?
        if [[ $ret_val -ne 0 ]]; then
            if [[ $cli_exists -eq 0 ]]; then
                sg_log 0 "ERROR: Could not access the 3PAR storage array ${RC3PAR_DC2_STORAGE_SYSTEM_NAME}. Neither Passwordless SSH is configured nor CLI is found."
                sg_log 0 "If Inform OS version is greater than 3.3.1, then Passwordless SSH needs to be configured."
                sg_log 0 "If Inform OS version is lesser than 3.3.1, then install CLI client and create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary."
                return 1
            else
                DC2_SSH_ERROR="ERROR: Passworldless SSH not set for ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} array with user ${RC3PAR_DC2_STORAGE_SYSTEM_USER} from system $host_nameWithoutDomain ."
                return 1 
            fi
        fi
    fi
}

#This functions finds if local node has the least
#node id among cluster nodes. It returs 0 if yes 
#else returns 1. This function is called so that
#some checks during validation happen only on one node

function find_if_local_node_has_least_node_id
{
    typeset node_ids=""
    typeset least_node=""
    typeset local_node_id=""
    typeset -i node_id=0
    typeset -i least_node_id=0
    cache_cmviewcl_output_lx
    if [[ $? -ne 0 ]]; then
        echo "ERROR: Unable to get cluster details"
        exit 1
    fi
    host_name=$(host=`hostname`; echo ${host%%.*})
    local_host_name=$(echo $host_name | awk -F"." '{print $1}');
    node_ids_line=`$SGSBIN/cmviewcl -vf line -s config | awk '/^node:[^|]*\|id=/'`
    local_node_id=`echo "$node_ids_line" | grep -w ${local_host_name} | awk -F= '/^node:[^|]*\|id=/ {print $2}'`
    least_node_id=$local_node_id
    least_node=$local_host_name
    package_node_names=`get_package_nodes`
    for node in $package_node_names
    do
        node_id=`echo "$node_ids_line" | grep -w $node | cut -d "=" -f 2`
        if [[ $? -eq 0 ]]; then
            if (( $node_id <= $least_node_id  ));then
                least_node_id=$node_id
                least_node=$node
            fi
        fi
    done

    if [[ "$least_node" == "$local_host_name" ]] ;then
        # True
        return 0
    else
        # False
        return 1
    fi
}

##This fucntion is to verify if the passowrd file for CLI 
##is usable or not by the CLi client.

function verify_pwdfile_cli
{
    if [[ "$1" = "DC1" ]]; then
       cmd=${cmd1}
       RC3PAR_DC_ST_SYS_NAME=${RC3PAR_DC1_STORAGE_SYSTEM_NAME}
       RC3PAR_DC_ST_SYS_USER=${RC3PAR_DC1_STORAGE_SYSTEM_USER}
    else
       cmd=${cmd2}
       RC3PAR_DC_ST_SYS_NAME=${RC3PAR_DC2_STORAGE_SYSTEM_NAME}
       RC3PAR_DC_ST_SYS_USER=${RC3PAR_DC2_STORAGE_SYSTEM_USER}
    fi

    if [[ ! -e "/usr/bin/cli" ]]; then
          sg_log 0 "ERROR: /usr/bin/cli not found"
          sg_log 0 "Create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary."
          to_exit=1
    elif [[ $(echo ${cmd} | grep "login failed") != "" ]]; then
          sg_log 0 "ERROR: Connection to the storage system ${RC3PAR_DC_ST_SYS_NAME} failed."
          sg_log 0 "This happens when either of the password file ${SGCONF}/3parrc/${RC3PAR_DC_ST_SYS_USER}_${RC3PAR_DC_ST_SYS_NAME}.pwf or password is modified or user is deleted from the storage system."
          sg_log 0 "If password file is modified and Inform OS version of ${RC3PAR_DC_ST_SYS_NAME} is lesser than 3.3.1 , recreate the password file for all nodes in the cluster."
          to_exit=1
    elif [[ $(echo ${cmd} | grep "Could not open") != "" ]]; then
          sg_log 0 "ERROR: The storage system $RC3PAR_DC_ST_SYS_NAME} is not reachable from ${host_name}."
          to_exit=1
    elif [[ $(echo ${cmd} | grep "invalid command name") != "" ]]; then
          #not doing anything in case of success
          dummy_variable_is_being_set=0
    else
          sg_log 0 "ERROR: Verification of CLI connection to ${RC3PAR_DC_ST_SYS_NAME} using password file ${SGCONF}/3parrc/${RC3PAR_DC_ST_SYS_USER}_${RC3PAR_DC_ST_SYS_NAME}.pwf failed due to" \"${cmd}\"
          to_exit=1
    fi

}

# This functions checks the array OS version
# and then sets whether CLI or ssh should be 
# used for that particular array.
function check_informOs_version
{
    typeset RM_version="3.3.1"
 
    if [[ $1 = "cli" ]]; then

        cmd1="/usr/bin/cli -sys ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf"
        informOSVersion1=`$cmd1 showversion -s 2>&1`
        rc=$?

        cmd1=`echo "${informOSVersion1}" | tr -d '\n\r'`
        if echo $cmd1 | grep "application-specific initialization failed" > /dev/null 2>&1; then 
            rc=1
            cli_corrupt_flag=true
        fi

        if [[ $rc -ne 0 ]]; then
          #Let us not check SSH for RH5
          if [[ $rh5 -eq 1 ]]; then
            DC1_to_exit=1
          else  
            check_ssh_connection DC1
            if [[ $? -eq 0 ]]; then
                informOSVersion1=`ssh ${RC3PAR_DC1_STORAGE_SYSTEM_USER}@${RC3PAR_DC1_STORAGE_SYSTEM_NAME} showversion -s 2>&1`
                check_ssh1_flag=true
            else
                DC1_to_exit=1
            fi
          fi
        else 
            # Got value using CLI
            check_cli1_flag=true
        fi

        if [[ "$DC1_to_exit" -eq 1 ]]; then
            sg_log 0 "ERROR: Failed to get InformOS version for $RC3PAR_DC1_STORAGE_SYSTEM_NAME"
            verify_pwdfile_cli DC1
            #Since we never tried, there will be no SSH error for RH5
            if [[ $rh5 -ne 1 ]]; then
              sg_log 0 $DC1_SSH_ERROR
              sg_log 0 "INFO: If Inform OS version for ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} is greater than 3.3.1, then Passwordless SSH needs to be configured."
            fi
        fi
        
               
        cmd2="/usr/bin/cli -sys ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf"
        informOSVersion2=`$cmd2 showversion -s 2>&1`
        rc2=$?
        cmd2=`echo "${informOSVersion2}" | tr -d '\n\r'`
        if echo $cmd2 | grep "application-specific initialization failed" > /dev/null 2>&1; then 
            rc2=1
            cli_corrupt_flag=true
        fi
        if [[ $rc2 -ne 0 ]]; then
          #Let us not check SSH for RH5
          if [[ $rh5 -eq 1 ]]; then
            DC2_to_exit=1
          else 
            check_ssh_connection DC2
            if [[ $? -eq 0 ]]; then
                informOSVersion2=`ssh ${RC3PAR_DC2_STORAGE_SYSTEM_USER}@${RC3PAR_DC2_STORAGE_SYSTEM_NAME} showversion -s 2>&1`
                check_ssh2_flag=true
            else 
                DC2_to_exit=1
            fi
          fi  
        else 
            # Got value using CLI
            check_cli1_flag=true
        fi

        if [[ "$DC2_to_exit" -eq 1 ]]; then
            sg_log 0 "ERROR: Failed to get InformOS version for $RC3PAR_DC2_STORAGE_SYSTEM_NAME."
            verify_pwdfile_cli DC2
            #Since we never tried, there will be no SSH error for RH5
            if [[ $rh5 -ne 1 ]]; then            
              sg_log 0 $DC2_SSH_ERROR
              sg_log 0 "INFO: If Inform OS version for ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} is greater than 3.3.1, then Passwordless SSH needs to be configured."
            fi
        fi

        ## Comeout by printing error message w.r.t CLI & SSH both
        if [ "$DC1_to_exit" -eq 1 ] || [ "$DC2_to_exit" -eq 1 ]; then
            return 1
        fi

    # check for $1=ssh
    elif [[ $1 = "ssh" ]]; then
      #We shall check SSH only if it is NOT RH5
      if [[ $rh5 -eq 1 ]]; then
        #We are here because CLI is not present, so let us print appropriate message for DC1 & DC2
        verify_pwdfile_cli DC1
        DC1_to_exit=1
      else   
        check_ssh_connection DC1
        if [[ $? -eq 0 ]]; then
            cmd1="ssh ${RC3PAR_DC1_STORAGE_SYSTEM_USER}@${RC3PAR_DC1_STORAGE_SYSTEM_NAME}"
            informOSVersion1=`$cmd1 showversion -s 2>&1`
            check_ssh1_flag=true
        else
            DC1_to_exit=1
        fi

        check_ssh_connection DC2
        if [[ $? -eq 0 ]]; then
            cmd2="ssh ${RC3PAR_DC2_STORAGE_SYSTEM_USER}@${RC3PAR_DC2_STORAGE_SYSTEM_NAME}"
            informOSVersion2=`$cmd2 showversion -s 2>&1`
            check_ssh2_flag=true
        else
           DC2_to_exit=1
        fi
      fi


        ## Comeout by printing error message w.r.t CLI & SSH both
        if [ "$DC1_to_exit" -eq 1 ] || [ "$DC2_to_exit" -eq 1 ]; then
            return 1
        fi

    # $1 value is neither cli nor ssh, so exit
    else
        sg_log 0 "ERROR: Not supported parameter has been passed."
        return 1
    fi

    # print the warning message only if the current node has the least node id
    if find_if_local_node_has_least_node_id; then
        if [[ "${informOSVersion1}" != "${informOSVersion2}" ]]; then
            sg_log 0 "WARNING: The InformOS version for ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} (${informOSVersion1}) and ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} (${informOSVersion2}) do not match."
            sg_log 0 "WARNING: Please refer 3par documentation for support information"
        fi
    fi    
           
    # Checks for informOS versions w.r.t to RM version 
    if [[ "${informOSVersion1}" = "$RM_version" ]] || [[ "${informOSVersion1}" > "$RM_version" ]]; then
      if [[ $rh5 -eq 1 ]]; then
        ssh_cli_cmd_dc1="/usr/bin/cli -sys ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf"
      else  
        if $check_cli1_flag; then
             check_ssh_connection DC1
             if [[ $? -ne 0 ]]; then
                 sg_log 0 $DC1_SSH_ERROR
                 DC1_to_exit=1
             fi
        fi
        if [[ ! -f ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf ]]; then
            sg_log 0 "ERROR: Please run the utility $SGSBIN/3parrc/setupMC3PARsetsshkeys with the parameter ${RC3PAR_DC1_STORAGE_SYSTEM_USER}@${RC3PAR_DC1_STORAGE_SYSTEM_NAME} and retry."
            DC1_to_exit=1
        fi
        ssh_cli_cmd_dc1="ssh ${RC3PAR_DC1_STORAGE_SYSTEM_USER}@${RC3PAR_DC1_STORAGE_SYSTEM_NAME}"
      fi   
    else
        if $check_ssh1_flag; then
             if $cli_corrupt_flag; then
                 RC=1
             else 
                 /usr/bin/cli -sys ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf showversion -s > /dev/null 2>&1
                 RC=$?
             fi
             if [[ $RC -ne 0 ]]; then
                 # print appropriate error message for the CLI & exit
                 verify_pwdfile_cli DC1
                 DC1_to_exit=1
             fi
        fi
        ssh_cli_cmd_dc1="/usr/bin/cli -sys ${RC3PAR_DC1_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC1_STORAGE_SYSTEM_USER}_${RC3PAR_DC1_STORAGE_SYSTEM_NAME}.pwf"
    fi
    
    if [[ "${informOSVersion2}" = "$RM_version" ]] || [[ "${informOSVersion2}" > "$RM_version" ]]; then
     if [[ $rh5 -eq 1 ]]; then
       ssh_cli_cmd_dc2="/usr/bin/cli -sys ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf"
     else
        if $check_cli2_flag; then
             check_ssh_connection DC2
             if [[ $? -ne 0 ]]; then
                 sg_log 0 $DC2_SSH_ERROR
                 DC2_to_exit=1
             fi
        fi
        if [[ ! -f ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf ]]; then
            sg_log 0 "ERROR: Please run the utility $SGSBIN/3parrc/setupMC3PARsetsshkeys with the parameter ${RC3PAR_DC2_STORAGE_SYSTEM_USER}@${RC3PAR_DC2_STORAGE_SYSTEM_NAME} and retry."
            DC2_to_exit=1
        fi
        ssh_cli_cmd_dc2="ssh ${RC3PAR_DC2_STORAGE_SYSTEM_USER}@${RC3PAR_DC2_STORAGE_SYSTEM_NAME}"
      fi
    else
        if $check_ssh2_flag; then
             if $cli_corrupt_flag; then
                 RC=1
             else 
                 /usr/bin/cli -sys ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf showversion -s > /dev/null 2>&1
                 RC=$?
             fi
             if [[ $RC -ne 0 ]]; then
                 # print appropriate error message for the CLI & exit
                 verify_pwdfile_cli DC2
                 DC2_to_exit=1
             fi
        fi
        ssh_cli_cmd_dc2="/usr/bin/cli -sys ${RC3PAR_DC2_STORAGE_SYSTEM_NAME} -pwf ${SGCONF}/3parrc/${RC3PAR_DC2_STORAGE_SYSTEM_USER}_${RC3PAR_DC2_STORAGE_SYSTEM_NAME}.pwf"
    fi

    if [ "$DC1_to_exit" -eq 1 ] || [ "$DC2_to_exit" -eq 1 ]; then
        return 1
    fi
    return 0
}

#######################################################################
#
# function sg_reconfigure_stop() is called by master control script
# during metrocluster 3parrc configuration validation.
#
#######################################################################

function sg_reconfigure_stop
{

   to_exit=0
   if sg_check_if_reconfig_has_failed
   then
      sg_log 999 "WARNING: mc3parrc - Not attempting to stop Remote Copy VG Monitor service."
      to_exit=1
      return $to_exit
   fi

   verify_service_restart
   service_restart_temp=$?
   if [[ $service_restart_temp -eq 2 ]];then
#exiting as online reconfiguration on SG version 11.20.10 or below will bring 
#down the package if attempted
    to_exit=0
   elif [[ $service_restart_temp -eq 1 ]];then
    sg_log 0 "Executing reconfigure stop for Remote Copy VG Monitor service."
    sg_log 0 "Detected no change in parameters related to Remote Copy VG Monitor service."
    to_exit=0
   else
    sg_log 0 "Executing reconfigure stop for Remote Copy VG Monitor service."
    for cmd in "${PREV_SG_SERVICE_CMD[@]}"
    do
        echo $cmd | grep "$SGSBIN/DR3PARRCMon $SG_PACKAGE_NAME" >> /dev/null
        if [[ $? -eq 0 ]]
             then
                service_name=${PREV_SG_SERVICE_NAME[$i]}
                break
             fi
             i=$i+1
       done
    if [[ "a${service_name}" != "a" ]];then
           ${SGSBIN}/cmhaltserv ${service_name}
                if [[ $? -ne 0 ]];then
                        sg_log 0 "ERROR: Unable to stop Remote Copy VG Monitor service."
                        sg_log 999 "ERROR: Unable to stop Remote Copy VG Monitor service."
                        sg_set_reconfig_failure
                        to_exit=1
                else
                        sg_log 0 "Successfully stopped Remote Copy VG Monitor service."
                        sg_log 999 "SUCCESS: Stopped Remote Copy VG Monitor service."
                        to_exit=0
                fi
       fi
   fi
   return $to_exit

}


#######################################################################
#
# function sg_reconfigure_start() is called by master control script
# during metrocluster 3parrc configuration validation.
#
#######################################################################

function sg_reconfigure_start
{
   to_exit=0

   verify_service_restart
   service_restart_temp=$?
   if [[ $service_restart_temp -eq 2 ]];then
#exiting as online reconfiguration on SG version 11.20.10 or below will bring 
#down the package if attempted
    to_exit=0
   elif [[ $service_restart_temp -eq 1 ]];then
    sg_log 0 "Executing reconfigure start for Remote Copy VG Monitor service."
    sg_log 0 "Detected no change in parameters related to Remote Copy VG Monitor service."
    to_exit=0
   else
    sg_log 0 "Executing reconfigure start for Remote Copy VG Monitor service."
    for cmd in "${SG_SERVICE_CMD[@]}"
    do
        echo $cmd | grep "$SGSBIN/DR3PARRCMon $SG_PACKAGE_NAME" >> /dev/null
        if [[ $? -eq 0 ]]
             then
                service_name=${SG_SERVICE_NAME[$i]}
            service_cmd=${SG_SERVICE_CMD[$i]}
                break
             fi
             i=$i+1
       done
    if [[ "a${service_name}" != "a" ]];then
        ${SGSBIN}/cmrunserv ${service_name} "$service_cmd"
                if [[ $? -ne 0 ]];then
                        sg_log 0 "ERROR: Unable to start Remote Copy VG Monitor service."
                        sg_log 999 "ERROR: Unable to start Remote Copy VG Monitor service."
                        sg_set_reconfig_failure
                        to_exit=1
                else
                        sg_log 0 "Successfully started Remote Copy VG Monitor service."
                        sg_log 999 "SUCCESS: Started Remote Copy VG Monitor service."
                        to_exit=0
                fi
       fi
   fi
   return $to_exit
}


############################################################
# Function to verify whether service restart is needed
# or not.
# Return values:
#   0 - if restart is needed
#   1 - if restart is not needed
#   2 - if sgversion is less than 11.20.10 online reconfig 
#       will bring down package
############################################################

function verify_service_restart
{
SORT=/bin/sort
    # Cluster verification and restricting online modification 
    # is only supported if Serviceguard version is
    # A.11.20.20 or later
    if [[ "$sg_version" > "11.20.20" || "$sg_version" = "11.20.20" ]];  then
            cur_email_id=( $(
                for list in "${SG_EMAIL_ID[@]}"
                   do
                      echo "$list"
                   done | $SORT) )

            prev_email_id=( $(
                   for list in "${PREV_SG_EMAIL_ID[@]}"
                   do
                      echo "$list"
                   done | $SORT) )
                                if [[ "${RC3PAR_DR_NOTIFICATION_CHOICES}" != "${PREV_RC3PAR_DR_NOTIFICATION_CHOICES}" || \
                                      "${cur_email_id[*]}" != "${prev_email_id[*]}" || \
                                      "${RC3PAR_DTS_PKG_DIR}" != "${PREV_RC3PAR_DTS_PKG_DIR}" || \
                                      "${RC3PAR_DC1_STORAGE_SYSTEM_NAME}" != "${PREV_RC3PAR_DC1_STORAGE_SYSTEM_NAME}" || \
                                      "${RC3PAR_DC2_STORAGE_SYSTEM_NAME}" != "${PREV_RC3PAR_DC2_STORAGE_SYSTEM_NAME}" || \
                                      "${RC3PAR_DC1_STORAGE_SYSTEM_USER}" != "${PREV_RC3PAR_DC1_STORAGE_SYSTEM_USER}" || \
                                      "${RC3PAR_DC2_STORAGE_SYSTEM_USER}" != "${PREV_RC3PAR_DC2_STORAGE_SYSTEM_USER}" || \
                                      "${RC3PAR_DC1_RC_TARGET_FOR_DC2}" != "${PREV_RC3PAR_DC1_RC_TARGET_FOR_DC2}" || \
                                      "${RC3PAR_DC2_RC_TARGET_FOR_DC1}" != "${PREV_RC3PAR_DC2_RC_TARGET_FOR_DC1}" || \
                                      "${RC3PAR_DC1_RC_VOLUME_GROUP}" != "${PREV_RC3PAR_DC1_RC_VOLUME_GROUP}" || \
                                      "${RC3PAR_DC2_RC_VOLUME_GROUP}" != "${PREV_RC3PAR_DC2_RC_VOLUME_GROUP}" ]]; then

                all_srv_restart=0
                else
                all_srv_restart=1
            fi
    else
       all_srv_restart=2
    fi
return $all_srv_restart
}

###############################################################################
# This function checks whether the volume manager is vxvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - vxvm is used
#          1 - vxvm is not used
#
###############################################################################


function  checkvxvmused
{
   if [[ ${SG_VXVM_DG[*]} != "" ]]
   then
      return 0
   fi
   return 1
}




################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#


typeset to_exit=0;

host_name=$(host=`hostname`; echo ${host%%.*})
host_nameWithoutDomain=$(echo $host_name | awk -F"." '{print $1}');
host_type="NA";

for system in ${RC3PAR_DC1_NODE_LIST}; do
    systemWithoutDomain=$(echo $system | sed 's/\"//g' | awk -F"." '{print $1}');
        if [[ ${systemWithoutDomain}  = ${host_nameWithoutDomain} ]]; then
                 host_type="DC1"
        fi
done
for system in ${RC3PAR_DC2_NODE_LIST}; do
    systemWithoutDomain=$(echo $system | sed 's/\"//g' | awk -F"." '{print $1}');
        if [[ ${systemWithoutDomain}  = ${host_nameWithoutDomain} ]]; then
                 host_type="DC2"
        fi
done

case ${1} in
       start)      sg_start_3parrc;;
        stop)      sg_stop_3parrc;;
    validate)      sg_validate_3parrc;;
    reconfigure_start) sg_reconfigure_start;;
    reconfigure_stop)  sg_reconfigure_stop;;
esac
#
# Exit to indicate success/failure to the master_control_script.sh
#

exit $to_exit


